@@ -8,6 +8,7 @@ import sys
 import weakref
 from collections import deque
 from enum import IntEnum
+from itertools import chain
 
 from .child import cwd_of_process
 from .config import build_ansi_color_table
@@ -98,6 +99,24 @@ def setup_colors(screen, opts):
         opts.foreground, opts.background, opts.cursor, opts.selection_foreground, opts.selection_background)))
 
 
+def text_sanitizer(as_ansi, add_wrap_markers):
+    import re
+    pat = re.compile(r'\033\[.+?m')
+
+    def remove_wrap_markers(line):
+        return line.replace('\r', '')
+
+    def remove_sgr(line):
+        return pat.sub('', line)
+
+    def remove_both(line):
+        return pat.sub('', line.replace('\r', ''))
+
+    if as_ansi:
+        return remove_both if add_wrap_markers else remove_sgr
+    return remove_wrap_markers
+
+
 class Window:
 
     def __init__(self, tab, child, opts, args, override_title=None):
@@ -248,7 +267,7 @@ class Window:
 
     # screen callbacks {{{
     def use_utf8(self, on):
-        get_boss().child_monitor.set_iutf8(self.window_id, on)
+        get_boss().child_monitor.set_iutf8(self.id, on)
 
     def focus_changed(self, focused):
         if focused:
@@ -376,6 +395,8 @@ class Window:
 
     def clipboard_control(self, data):
         where, text = data.partition(';')[::2]
+        if not where:
+            where = 's0'
         if text == '?':
             response = None
             if 's' in where or 'c' in where:
@@ -436,15 +457,22 @@ class Window:
             self.screen.reset_callbacks()
         self.screen = None
 
-    def as_text(self, as_ansi=False, add_history=False, add_wrap_markers=False):
+    def as_text(self, as_ansi=False, add_history=False, add_wrap_markers=False, alternate_screen=False):
         lines = []
-        add_history = add_history and not self.screen.is_using_alternate_linebuf()
-        f = self.screen.as_text_non_visual if add_history else self.screen.as_text
+        add_history = add_history and not (self.screen.is_using_alternate_linebuf() ^ alternate_screen)
+        if alternate_screen:
+            f = self.screen.as_text_alternate
+        else:
+            f = self.screen.as_text_non_visual if add_history else self.screen.as_text
         f(lines.append, as_ansi, add_wrap_markers)
         if add_history:
             h = []
+            self.screen.historybuf.pagerhist_as_text(h.append)
+            if not as_ansi or not add_wrap_markers:
+                sanitizer = text_sanitizer(as_ansi, add_wrap_markers)
+                h = list(map(sanitizer, h))
             self.screen.historybuf.as_text(h.append, as_ansi, add_wrap_markers)
-            lines = h + lines
+            lines = chain(h, lines)
         return ''.join(lines)
 
     @property
@@ -455,15 +483,26 @@ class Window:
         if pid is not None:
             return cwd_of_process(pid) or None
 
+    def pipe_data(self, text, has_wrap_markers=False):
+        text = text or ''
+        if has_wrap_markers:
+            text = text.replace('\r\n', '\n').replace('\r', '\n')
+        lines = text.count('\n')
+        input_line_number = (lines - (self.screen.lines - 1) - self.screen.scrolled_by)
+        return {
+            'input_line_number': input_line_number, 'scrolled_by': self.screen.scrolled_by,
+            'cursor_x': self.screen.cursor.x + 1, 'cursor_y': self.screen.cursor.y + 1,
+            'lines': self.screen.lines, 'columns': self.screen.columns,
+            'text': text
+        }
+
     # actions {{{
 
     def show_scrollback(self):
-        data = self.as_text(as_ansi=True, add_history=True, add_wrap_markers=True)
-        data = data.replace('\r\n', '\n').replace('\r', '\n')
-        lines = data.count('\n')
-        input_line_number = (lines - (self.screen.lines - 1) - self.screen.scrolled_by)
-        cmd = [x.replace('INPUT_LINE_NUMBER', str(input_line_number)) for x in self.opts.scrollback_pager]
-        get_boss().display_scrollback(self, data, cmd)
+        text = self.as_text(as_ansi=True, add_history=True, add_wrap_markers=True)
+        data = self.pipe_data(text, has_wrap_markers=True)
+        cmd = [x.replace('INPUT_LINE_NUMBER', str(data['input_line_number'])) for x in self.opts.scrollback_pager]
+        get_boss().display_scrollback(self, data['text'], cmd)
 
     def paste(self, text):
         if text and not self.destroyed:
@@ -475,6 +514,10 @@ class Window:
                     if len(text) == len(new_text):
                         break
                     text = new_text
+            else:
+                # Workaround for broken editors like nano that cannot handle
+                # newlines in pasted text see https://github.com/kovidgoyal/kitty/issues/994
+                text = b'\r'.join(text.splitlines())
             self.screen.paste(text)
 
     def copy_to_clipboard(self):
